head	1.9;
access;
symbols;
locks; strict;
comment	@ * @;


1.9
date	2000.11.27.23.08.35;	author vlefevre;	state Exp;
branches;
next	1.8;

1.8
date	2000.11.27.15.06.40;	author lefevre;	state Exp;
branches;
next	1.7;

1.7
date	2000.11.23.11.26.38;	author lefevre;	state Exp;
branches;
next	1.6;

1.6
date	2000.11.23.11.22.16;	author lefevre;	state Exp;
branches;
next	1.5;

1.5
date	2000.11.22.17.23.31;	author lefevre;	state Exp;
branches;
next	1.4;

1.4
date	2000.11.22.01.49.29;	author vlefevre;	state Exp;
branches;
next	1.3;

1.3
date	2000.11.21.15.45.51;	author lefevre;	state Exp;
branches;
next	1.2;

1.2
date	2000.11.21.15.18.21;	author lefevre;	state Exp;
branches;
next	1.1;

1.1
date	2000.11.21.01.16.57;	author vlefevre;	state Exp;
branches;
next	;


desc
@Multiplication by a Constant -- Bernstein's Algorithm
@


1.9
log
@Pruning improved.
@
text
@/*
 * Multiplication by a Constant -- Bernstein's Algorithm
 *
 * Usage: bernstein <mode> [ <constant> ... ]
 *
 * Compile with -DPRUNE to prune the tree.
 * Compile with -DNCALLS to get the number of get_node() and try() calls.
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>


#ifdef LONGLONG
typedef unsigned long long int VALUE;
#define VALUEFMT "llu"
#define STRTOVALUE(S) strtoull(S, (char **) NULL, 0)
#else
typedef unsigned long int VALUE;
#define VALUEFMT "lu"
#define STRTOVALUE(S) strtoul(S, (char **) NULL, 0)
#endif

#ifndef ADD_COST
#define ADD_COST 1
#endif

#ifndef SUB_COST
#define SUB_COST 1
#endif

#ifndef SHIFT_COST
#define SHIFT_COST 0
#endif

#ifndef HASH_SIZE
#define HASH_SIZE 16383
#endif

#ifndef MAXNON
#define MAXNON 65536
#endif

#ifdef PRUNE
#define PLIMIT , &limit
#else
#define PLIMIT
#endif


#define odd(n) ((n) & 1)
#define even(n) (!odd(n))

enum { EXIT_OK, EXIT_MEMORY, EXIT_USAGE, EXIT_BADMODE, EXIT_BADCONST,
       EXIT_INTERROR, EXIT_OVERFLOW };

typedef enum { INVALID, NOOP, ADD1, SUB1, FADD, FSUB } OP;
static char opsign[] = { ' ', ' ', '+', '-', '+', '-' };

typedef struct node {
  struct node *parent;  /* node used to generate the current node  */
  VALUE value;          /* value of the current node (odd integer) */
  unsigned int cost;    /* cost of the value, or lower bound       */
  OP opcode;            /* operation                               */
  unsigned int shift;   /* shift count                             */
  struct node *next;    /* next node having the same hash code     */
} NODE;


static NODE *hash_table[HASH_SIZE];
static long int non = 0;  /* number of nodes */
static int mode;
#ifdef NCALLS
static unsigned long int ngn = 0, ntry = 0, nmalloc = 0;
#endif

void init_hash(void);
VALUE get_cst(char *s);
#ifdef PRUNE
NODE *get_node(VALUE n, unsigned int limit);
void try(VALUE n, NODE *node, OP opcode,
    unsigned int cost, unsigned int shift, unsigned int *limit);
#else
NODE *get_node(VALUE n);
void try(VALUE n, NODE *node, OP opcode,
    unsigned int cost, unsigned int shift);
#endif
void bernstein(VALUE n);
unsigned int emit_code(NODE *node);


int main(int argc, char **argv)
{
  if (argc < 2)
  {
    fprintf(stderr, "Usage: bernstein <mode> [ <constant> ... ]\n");
    exit(EXIT_USAGE);
  }

  mode = atoi(argv[1]);
  if (errno)
  {
    fprintf(stderr, "bernstein: bad mode\n");
    exit(EXIT_BADMODE);
  }

  init_hash();

  if (argc > 2)
  {
    unsigned int i;
    for (i = 2; i < argc; i++)
      bernstein(get_cst(argv[i]));
  }
  else
  {
    char buffer[64];
    while (scanf("%63s", buffer) == 1)
      bernstein(get_cst(buffer));
  }

  return EXIT_OK;
}


void init_hash(void)
{
  unsigned int i;
  for (i = 0; i < HASH_SIZE; i++)
  {
    NODE *node;
    node = hash_table[i];
    while (node)
    {
      NODE *next;
      next = node->next;
      if (non-- == 0)
      {
        fprintf(stderr, "bernstein: internal error ('non' too low)!\n");
        exit(EXIT_INTERROR);
      }
      free(node);
      node = next;
    }
    hash_table[i] = NULL;
  }
  if (non == 0) return;
  fprintf(stderr, "bernstein: internal error ('non' too high)!\n");
  exit(EXIT_INTERROR);
}


VALUE get_cst(char *s)
{
  VALUE n;
  n = STRTOVALUE(s);
  if (errno || n == 0 || even(n))
  {
    fprintf(stderr, "bernstein: bad constant '%s'\n", s);
    exit(EXIT_BADCONST);
  }
  return n;
}


#ifdef PRUNE
NODE *get_node(VALUE n, int unsigned limit)
#else
NODE *get_node(VALUE n)
#endif
{
  unsigned int hash;
  NODE *node;

#ifdef NCALLS
  if (++ngn == 0)
  {
    fprintf(stderr, "bernstein: ngn overflow\n");
    exit(EXIT_OVERFLOW);
  }
#endif

  if (mode >= 2)
#ifdef PRUNE
    printf("get_node %" VALUEFMT " %u\n", n, limit);
#else
    printf("get_node %" VALUEFMT "\n", n);
#endif

  hash = n % HASH_SIZE;
  node = hash_table[hash];

  while (node)
  {
    if (node->value == n)
    {
#ifdef PRUNE
      if (node->opcode == INVALID && node->cost <= limit)
        goto validate_node;
#endif
      return node;
    }
    node = node->next;
  }

#ifdef NCALLS
  if (++nmalloc == 0)
  {
    fprintf(stderr, "bernstein: nmalloc overflow\n");
    exit(EXIT_OVERFLOW);
  }
#endif
  node = malloc(sizeof *node);
  if (!node)
  {
    fprintf(stderr, "bernstein: out of memory (%ld nodes)!\n", non);
    exit(EXIT_MEMORY);
  }
  non++;
  node->parent = NULL;
  node->value = n;
  node->next = hash_table[hash];
  hash_table[hash] = node;
#ifdef PRUNE
  node->opcode = INVALID;
validate_node:
#endif

  if (n == 1)
  {
    node->cost = 0;
    node->opcode = NOOP;
  }
  else
  {
    VALUE d = 4, dsup;
    int shift = 2;
    dsup = n >> 1;
#ifdef PRUNE
    node->cost = limit + 1;  /* Lower bound on the cost in case the */
                             /* following calls to try would fail.  */
#endif
    while (d <= dsup)
    {
      if (n % (d - 1) == 0)
        try(n / (d - 1), node, FSUB, SHIFT_COST + SUB_COST, shift PLIMIT);
      if (n % (d + 1) == 0)
        try(n / (d + 1), node, FADD, SHIFT_COST + ADD_COST, shift PLIMIT);
      d <<= 1;
      shift++;
    }
    try(n - 1, node, ADD1, SHIFT_COST + ADD_COST, 0 PLIMIT);
    try(n + 1, node, SUB1, SHIFT_COST + SUB_COST, 0 PLIMIT);
  }

  return node;
}


#ifdef PRUNE
void try(VALUE n, NODE *node, OP opcode,
    unsigned int cost, unsigned int shift, unsigned int *limit)
#else
void try(VALUE n, NODE *node, OP opcode,
    unsigned int cost, unsigned int shift)
#endif
{
  NODE *tmp_node;

#ifdef NCALLS
  if (++ntry == 0)
  {
    fprintf(stderr, "bernstein: ntry overflow\n");
    exit(EXIT_OVERFLOW);
  }
#endif

  while(even(n))
  {
    n >>= 1;
    shift++;
  }

#ifdef PRUNE
  if (cost > *limit) return;
  tmp_node = get_node(n, *limit - cost);
  if (tmp_node->opcode == INVALID) return;
#else
  tmp_node = get_node(n);
#endif

  cost += tmp_node->cost;
#ifdef PRUNE
  if (cost > *limit) return;
#endif
  if (!node->parent || cost < node->cost)
  {
    node->parent = tmp_node;
    node->cost = cost;
    node->opcode = opcode;
    node->shift = shift;
#ifdef PRUNE
    *limit = cost - 1;
#endif
    if (mode >= 2)
      printf("node %" VALUEFMT ": parent %" VALUEFMT ", opcode %d, "
             "shift count %u, cost %u\n", node->value,
             node->parent->value, node->opcode, node->shift, node->cost);
  }
}

void bernstein(VALUE n)
{
  NODE *node;
#ifdef PRUNE
  VALUE p;
  unsigned int limit = 0;
#endif

  if (non > MAXNON)
    init_hash();

#ifdef PRUNE
  p = n >> 1;
  while (p)  /* count the number of 1's in p */
  {
    limit += SHIFT_COST + ADD_COST;
    p &= p-1;
  }
  node = get_node(n, limit);
#else
  node = get_node(n);
#endif
  printf("Cost(%" VALUEFMT ") = %u\n", n, node->cost);
  if (mode)
    emit_code(node);
#ifdef NCALLS
  printf("%lu calls to get_node()\n", ngn);
  printf("%lu calls to try()\n", ntry);
  printf("%lu calls to malloc()\n", nmalloc);
#endif
  fflush(stdout);
}

unsigned int emit_code(NODE *node)
{
  unsigned int i;

  if (node == NULL)
    return 0;

  i = emit_code(node->parent);
  printf("u%u = ", i);
  if (node->opcode == NOOP)
  {
    printf("1");
  }
  else
  {
    printf("u%u << %u %c ", i-1, node->shift, opsign[node->opcode]);
    if (node->opcode == ADD1 || node->opcode == SUB1)
      printf("1");
    else
      printf("u%u", i-1);
  }
  printf("\n");

  return i+1;
}


/* $Id: bernstein.c 1.8 2000/11/27 15:06:40 lefevre Exp vlefevre $ */
@


1.8
log
@Mode 2.
Bug fixed in pruning.
NCALLS support.
@
text
@d64 1
a64 1
  unsigned int cost;    /* cost of the value                       */
d199 2
a200 1
      if (node->opcode == INVALID) goto validate_node;
d240 4
a243 1

d373 1
a373 1
/* $Id: bernstein.c 1.7 2000/11/23 11:26:38 lefevre Exp lefevre $ */
@


1.7
log
@Bug fixed.
@
text
@d5 3
d56 1
a56 1
       EXIT_INTERROR };
d74 3
d176 15
d206 7
d267 8
d290 3
d302 4
d334 5
d369 1
a369 1
/* $Id: bernstein.c 1.6 2000/11/23 11:22:16 lefevre Exp lefevre $ */
@


1.6
log
@HASH_SIZE increased.
Maximum number of nodes in order to avoid 'out of memory' errors.
@
text
@d130 2
a131 2
      NODE *parent;
      parent = node->parent;
d138 1
a138 1
      node = parent;
d321 1
a321 1
/* $Id: bernstein.c 1.5 2000/11/22 17:23:31 lefevre Exp lefevre $ */
@


1.5
log
@Optional pruning.
@
text
@d35 5
a39 1
#define HASH_SIZE 511
d52 2
a53 1
enum { EXIT_OK, EXIT_MEMORY, EXIT_USAGE, EXIT_BADMODE, EXIT_BADCONST };
d69 1
d125 15
d141 4
d188 1
a188 1
    fprintf(stderr, "bernstein: out of memory!\n");
d191 1
d272 6
d321 1
a321 1
/* $Id: bernstein.c 1.4 2000/11/22 01:49:29 vlefevre Exp lefevre $ */
@


1.4
log
@Flush stdout before reading next value (useful with a double pipe).
@
text
@d38 6
d50 2
a51 2
typedef enum { NOOP, ADD1, SUB1, FADD, FSUB } OP;
static char opsign[] = { ' ', '+', '-', '+', '-' };
d68 5
a73 1
NODE *make_node(VALUE n);
d76 1
d136 3
d140 1
d151 4
d156 1
a159 11
  node = make_node(n);
  node->next = hash_table[hash];
  hash_table[hash] = node;
  return node;
}


NODE *make_node(VALUE n)
{
  NODE *node;

d168 6
d185 1
d189 1
a189 1
        try(n / (d - 1), node, FSUB, SHIFT_COST + SUB_COST, shift);
d191 1
a191 1
        try(n / (d + 1), node, FADD, SHIFT_COST + ADD_COST, shift);
d195 2
a196 2
    try(n - 1, node, ADD1, SHIFT_COST + ADD_COST, 0);
    try(n + 1, node, SUB1, SHIFT_COST + SUB_COST, 0);
d203 4
d209 1
d219 5
d225 2
d234 3
d243 11
a253 1

d255 1
d289 1
a289 1
/* $Id: bernstein.c 1.3 2000/11/21 15:45:51 lefevre Exp vlefevre $ */
@


1.3
log
@Mode 1 added (emitted code).
@
text
@d217 1
d247 1
a247 1
/* $Id: bernstein.c 1.2 2000/11/21 15:18:21 lefevre Exp lefevre $ */
@


1.2
log
@Mode and multiple arguments added.
Arguments can be given via stdin.
@
text
@d45 1
d58 1
a58 1
int mode;
d64 2
a65 1
void try(VALUE n, NODE *node, OP opcode, unsigned int cost);
d67 1
d168 1
d173 1
a173 1
        try(n / (d - 1), node, FSUB, SHIFT_COST + SUB_COST);
d175 1
a175 1
        try(n / (d + 1), node, FADD, SHIFT_COST + ADD_COST);
d177 1
d179 2
a180 2
    try(n - 1, node, ADD1, SHIFT_COST + ADD_COST);
    try(n + 1, node, SUB1, SHIFT_COST + SUB_COST);
d187 2
a188 1
void try(VALUE n, NODE *node, OP opcode, unsigned int cost)
a190 1
  unsigned int shift = 0;
d215 28
d246 1
a246 1
/* $Id: bernstein.c 1.1 2000/11/21 01:16:57 vlefevre Exp lefevre $ */
@


1.1
log
@Initial revision
@
text
@d4 1
a4 1
 * Usage: bernstein <constant>
d14 2
a15 1
#define STRTOVALUE(S) strtoull(S, (void *) NULL, 0)
d18 2
a19 1
#define STRTOVALUE(S) strtoul(S, (void *) NULL, 0)
d42 2
d57 1
d60 1
d69 1
a69 3
  VALUE n;

  if (argc != 2)
d71 2
a72 2
    fprintf(stderr, "Usage: bernstein <constant>\n");
    exit(1);
d75 2
a76 2
  n = STRTOVALUE(argv[1]);
  if (errno || n == 0 || even(n))
d78 2
a79 2
    fprintf(stderr, "bernstein: bad constant '%s'\n", argv[1]);
    exit(2);
a82 1
  bernstein(n);
d84 14
a97 1
  return 0;
d109 13
d152 1
a152 1
    exit(3);
d209 1
a209 1
  printf("Cost = %u\n", node->cost);
d213 1
a213 1
/* $Id$ */
@
