head	1.14;
access;
symbols;
locks; strict;
comment	@ * @;


1.14
date	2002.12.31.20.56.11;	author lefevre;	state Exp;
branches;
next	1.13;

1.13
date	2002.12.28.03.03.37;	author lefevre;	state Exp;
branches;
next	1.12;

1.12
date	2002.12.19.13.54.02;	author lefevre;	state Exp;
branches;
next	1.11;

1.11
date	2002.12.17.02.17.03;	author lefevre;	state Exp;
branches;
next	1.10;

1.10
date	2002.12.12.09.17.09;	author lefevre;	state Exp;
branches;
next	1.9;

1.9
date	2001.05.04.08.04.11;	author lefevre;	state Exp;
branches;
next	1.8;

1.8
date	2001.05.04.00.26.33;	author lefevre;	state Exp;
branches;
next	1.7;

1.7
date	2001.05.03.13.07.12;	author lefevre;	state Exp;
branches;
next	1.6;

1.6
date	2001.05.02.02.51.28;	author lefevre;	state Exp;
branches;
next	1.5;

1.5
date	2001.05.02.02.49.32;	author lefevre;	state Exp;
branches;
next	1.4;

1.4
date	2001.04.30.08.53.40;	author lefevre;	state Exp;
branches;
next	1.3;

1.3
date	2001.04.30.04.15.25;	author lefevre;	state Exp;
branches;
next	1.2;

1.2
date	2001.04.25.12.17.03;	author lefevre;	state Exp;
branches;
next	1.1;

1.1
date	2001.04.19.13.54.55;	author lefevre;	state Exp;
branches;
next	;


desc
@Recherche de calculs possibles sur un ensemble de DAGs.
@


1.14
log
@Correction mineure: "(%7[(),0-9])" -> "(%7[,0-9])".
@
text
@/*
 * $Id: dagsearch.c 1.13 2002/12/28 03:03:37 lefevre Exp lefevre $
 *
 * Usage: dagsearch <mrec> <mmax> <cost> [ <cinf> <csup> <code> ]
 *   mrec: maximum recorded value
 *   mmax: maximum considered value in the search
 *   cost: file where the costs are saved
 *   cinf: file where the cinf values are saved
 *   csup: file where the csup values are saved
 *   code: file where the codes corresponding to cinf and csup are saved
 *   standard input: list of DAGs (e.g. given by gendags)
 *
 * Define SHIFTS to compute cinf and csup.
 *
 * In practice, searches will be performed with DAGs whose length is
 * not larger than 6. Thus, a code can be encoded in 8 bytes: 2 bytes
 * for the DAG number and one byte for each elementary operation: for
 * each operation, the type of the operation is encoded in the most
 * significant 2 bits and the shift count (assumed to be <= 63) is
 * stored in the other 6 bits. When the shift count is 0, the shifted
 * value is assumed to be the second one; thus the byte corresponding
 * to an operation is never 0. Therefore, the sequences shorter than
 * 8 bytes can be padded with null bytes.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define QMAX 32

#ifdef LONGLONG
typedef unsigned long long int VALUE;
#define VALUEFMT "llu"
#define STRTOVALUE(S) strtoull(S, (char **) NULL, 0)
#else
typedef unsigned long int VALUE;
#define VALUEFMT "lu"
#define STRTOVALUE(S) strtoul(S, (char **) NULL, 0)
#endif

#ifdef NDEBUG
#define OUT(...)
#else
#define OUT(...) printf(__VA_ARGS__)
#endif

#define DIST(x,y) ((x) >= (y) ? (x) - (y) : (y) - (x))

#ifdef SHIFTS
void stcode(unsigned char *code, int *o, int *r, int *s,
            unsigned int line, int q)
{
  int i;

  code[0] = (unsigned char) line;
  code[1] = (unsigned char) (line >> 8);
  code++;
  for (i = 1; i <= q; i++)
    *(++code) = (o[i] << 7) | (r[i] << 6) | s[i];
  while (i++ <= 6)
    *(++code) = 0;
}
#endif

void dagsearch(int q, int *dag, unsigned char *cost,
#ifdef SHIFTS
               unsigned char *cinf,
               unsigned char *csup,
               unsigned char *code,
               unsigned int line,
#endif
               long mrec, long mmax)
{
  int i;
  int *o, *r, *s;
  VALUE *x;
  int op[QMAX];  /* 0: add; 1: sub */
  int rs[QMAX];  /* 0: shift on the 1st value,
                    1: shift on the 2nd value */
  int sh[QMAX];  /* shift count */
  VALUE v[QMAX+1];  /* values */
  VALUE vp[2*QMAX];

  printf("DAG [ ");
  for (i = 0; i < q; i++)
    printf("(%d,%d) ", dag[2*i], dag[2*i+1]);
  printf("]\n");
  fflush(stdout);

  o = op - 1;
  r = rs - 1;
  s = sh - 1;
  x = vp - 2;
  dag -= 2;
  v[0] = 1;
  for (i = 0; ;)
    {
      if (i == q)
        {
        next:
          s[i]++;
          x[2*i+r[i]] <<= 1;
          v[i] = o[i] ? DIST(x[2*i], x[2*i+1]) : x[2*i] + x[2*i+1];
        }
      else
        {
          i++;
          x[2*i] = v[dag[2*i]];
          x[2*i+1] = v[dag[2*i+1]];
          o[i] = 0;
          r[i] = 1;
          s[i] = 0;
          v[i] = x[2*i] + x[2*i+1];
        }
      while (v[i] > mmax)
        {
          x[2*i] = v[dag[2*i]];
          x[2*i+1] = v[dag[2*i+1]];
          if (r[i] && s[i])
            {
              x[2*i] <<= 1;
              r[i] = 0;
              s[i] = 1;
              v[i] = o[i] ? DIST(x[2*i], x[2*i+1]) : x[2*i] + x[2*i+1];
            }
          else if (o[i] == 0)
            {
              o[i] = 1;
              r[i] = 1;
              s[i] = 0;
              v[i] = DIST(x[2*i], x[2*i+1]);
            }
          else if (--i)
            goto next;
          else
            return;
        }
      if (v[i] == 0)
        goto next;
      if (i < q)
        {
          int j;
          for (j = 0; j < i; j++)
            {
              VALUE x;
              x = v[j];
              while (x < v[i])
                x <<= 1;
              if (x == v[i])
                goto next;
            }
        }
      OUT("%3d: %3" VALUEFMT "  ( %3" VALUEFMT " %3" VALUEFMT " )",
          i, v[i], x[2*i], x[2*i+1]);
      if (v[i] <= mrec)
        {
          if (cost[v[i]] > i)
            {
              cost[v[i]] = i;
              OUT("   -   Cost(%3" VALUEFMT ") = %d", v[i], i);
#ifdef SHIFTS
              if (v[i] & 1)
                {
                  unsigned int cmax = 0;
                  int j;
                  for (j = 0; j < i; j++)
                    if (sh[j] > cmax)
                      cmax = sh[j];
                  cinf[v[i]] = cmax;
                  csup[v[i]] = cmax;
                  stcode(code + (v[i]<<3) - 8, o, r, s, line, i);
                  stcode(code + (v[i]<<3), o, r, s, line, i);
                }
#endif
            }
#ifdef SHIFTS
          if ((v[i] & 1) && cost[v[i]] == i)
            {
              unsigned int cmax = 0;
              int j;
              for (j = 0; j < i; j++)
                if (sh[j] > cmax)
                  cmax = sh[j];
              if (cinf[v[i]] > cmax)
                {
                  cinf[v[i]] = cmax;
                  stcode(code + (v[i]<<3) - 8, o, r, s, line, i);
                }
              if (csup[v[i]] < cmax)
                {
                  csup[v[i]] = cmax;
                  stcode(code + (v[i]<<3), o, r, s, line, i);
                }
            }
#endif
        }
      OUT("\n");
    }
}

void save(unsigned char *array, VALUE size, char *filename)
{
  FILE *f;

  f = fopen(filename, "wb");
  if (f == NULL)
    {
      fprintf(stderr, "dagsearch: cannot create file %s\n", filename);
      exit(11);
    }
  if (fwrite(array, size, 1, f) != 1)
    {
      fprintf(stderr, "dagsearch: cannot write to file %s\n", filename);
      exit(12);
    }
  if (fclose(f))
    {
      fprintf(stderr, "dagsearch: cannot close file %s\n", filename);
      exit(13);
    }
}

int main(int argc, char **argv)
{
  VALUE mrec, mmax;
  unsigned int line = 0;
  unsigned char *cost;
#ifdef SHIFTS
  unsigned char *cinf, *csup, *code;
#endif

#ifndef SHIFTS
  if (argc != 4)
    {
      fprintf(stderr, "Usage: dagsearch <mrec> <mmax> <file>\n");
      exit(1);
    }
#else
  if (argc != 7)
    {
      fprintf(stderr, "Usage: dagsearch <mrec> <mmax>"
              " <cost> <cinf> <csup> <code>\n");
      exit(1);
    }
#endif

  mrec = STRTOVALUE(argv[1]);
  if (errno == ERANGE)
    {
      fprintf(stderr, "dagsearch: mrec is too large (out of range)\n");
      exit(3);
    }
  if (mrec < 1)
    {
      fprintf(stderr, "dagsearch: mrec must be at least 1\n");
      exit(2);
    }

  mmax = STRTOVALUE(argv[2]);
  if (errno == ERANGE)
    {
      fprintf(stderr, "dagsearch: mmax is too large (out of range)\n");
      exit(5);
    }
  if (mmax < mrec)
    {
      fprintf(stderr, "dagsearch: mmax must be greater or equal to mrec\n");
      exit(4);
    }

  cost = malloc(mrec+1);
  if (cost == NULL)
    {
      fprintf(stderr, "dagsearch: out of memory!\n");
      exit(10);
    }
  memset(cost, -1, mrec+1);
  cost[0] = 0;
  cost[1] = 0;

#ifdef SHIFTS
  cinf = malloc(mrec+1);
  csup = malloc(mrec+1);
  code = malloc((mrec+1) << 3);
  if (cinf == NULL || csup == NULL || code == NULL)
    {
      fprintf(stderr, "dagsearch: out of memory!\n");
      exit(10);
    }
  cinf[0] = csup[0] = 0;
  cinf[1] = csup[1] = 0;
  memset(code, 0, 16);
#endif

  while (1)
    {
      VALUE v;
      int q;
      int dag[2*QMAX];
      char buffer[8];

      line++;
      for (q = 0; scanf("%1[\n]", buffer) == 0; q++)
        {
          if (q > QMAX)
            {
              fprintf(stderr, "dagsearch: too many nodes at line %u\n",
                      line);
              exit(6);
            }
          if (scanf("(%7[,0-9])", buffer) != 1 ||
              sscanf(buffer, "%d,%d", dag+2*q, dag+2*q+1) != 2)
            {
              fprintf(stderr, "dagsearch: input error at line %u\n", line);
              exit(7);
            }
          scanf("%*[\t ]");
        }

      if (!q)
        {
          if (feof(stdin))
            return 0;
          fprintf(stderr, "dagsearch: input error at line %u\n", line);
          exit(8);
        }

      dagsearch(q, dag, cost,
#ifdef SHIFTS
                cinf, csup, code, line,
#endif
                mrec, mmax);

      for (v = 1; v < mrec; v += 2)
        {
          VALUE w;
          int c;

          c = cost[v];
          w = v;
          while ((w <<= 1) <= mrec)
            if (cost[w] > c)
              {
                cost[w] = c;
#ifdef SHIFTS
                cinf[w] = csup[w] = 0;
#endif
              }
        }

      save(cost, mrec+1, argv[3]);
#ifdef SHIFTS
      save(cinf, mrec+1, argv[4]);
      save(csup, mrec+1, argv[5]);
      save(code, (mrec+1) << 3, argv[6]);
#endif
    }
}
@


1.13
log
@Si compil avec -DSHIFTS, sauvegarde du code correspondant  cinf et csup.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.12 2002/12/19 13:54:02 lefevre Exp lefevre $
d313 1
a313 1
          if (scanf("(%7[(),0-9])", buffer) != 1 ||
@


1.12
log
@Petite modification: on commence par dcaler le second fils et le cas
c_i = 0 apparatra comme si le second fils tait dcal. Ainsi, o[i],
r[i] et s[i] ne peuvent plus tre simultanment nuls, et le triplet
(0,0,0) pourra tre un marqueur de fin dans les futures extensions.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.11 2002/12/17 02:17:03 lefevre Exp lefevre $
d4 1
a4 1
 * Usage: dagsearch <mrec> <mmax> <cost> [ <cinf> <csup> ]
d10 1
d14 10
d51 16
d69 4
a72 1
               unsigned char *cinf, unsigned char *csup,
d164 12
a175 9
              {
                unsigned int cmax = 0;
                int j;
                for (j = 0; j < i; j++)
                  if (sh[j] > cmax)
                    cmax = sh[j];
                cinf[v[i]] = cmax;
                csup[v[i]] = cmax;
              }
d179 1
a179 1
          if (cost[v[i]] == i)
d187 4
a190 1
                cinf[v[i]] = cmax;
d192 4
a195 1
                csup[v[i]] = cmax;
d228 1
a228 1
  unsigned long line = 0;
d231 1
a231 1
  unsigned char *cinf, *csup;
d241 1
a241 1
  if (argc != 6)
d243 2
a244 2
      fprintf(stderr,
              "Usage: dagsearch <mrec> <mmax> <cost> <cinf> <csup>\n");
d286 2
a287 1
  if (cinf == NULL || csup == NULL)
d294 1
d309 1
a309 1
              fprintf(stderr, "dagsearch: too many nodes at line %lu\n",
d316 1
a316 1
              fprintf(stderr, "dagsearch: input error at line %lu\n", line);
d326 1
a326 1
          fprintf(stderr, "dagsearch: input error at line %lu\n", line);
d332 1
a332 1
                cinf, csup,
d357 1
@


1.11
log
@Possibilit de calculer cinf et csup en compilant avec -DSHIFTS.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.10 2002/12/12 09:17:09 lefevre Exp lefevre $
d83 1
a83 1
          r[i] = 0;
d91 1
a91 1
          if (r[i] == 0 && s[i])
d93 2
a94 2
              x[2*i+1] <<= 1;
              r[i] = 1;
d101 1
a101 1
              r[i] = 0;
@


1.10
log
@Style GNU + changement des messages d'erreur.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.9 2001/05/04 08:04:11 lefevre Exp lefevre $
d4 1
a4 1
 * Usage: dagsearch <mrec> <mmax> <file>
d7 3
a9 1
 *   file: file where the values are saved
d12 1
d40 5
a44 1
void dagsearch(int q, int *dag, unsigned char *cost, long mrec, long mmax)
d127 1
a127 1
      if (v[i] <= mrec && cost[v[i]] > i)
d129 30
a158 2
          cost[v[i]] = i;
          OUT("   -   Cost(%3" VALUEFMT ") = %d", v[i], i);
d164 22
d191 3
d195 1
d201 8
d244 12
a261 1
      FILE *f;
d289 5
a293 1
      dagsearch(q, dag, cost, mrec, mmax);
d304 6
a309 1
              cost[w] = c;
d312 5
a316 16
      f = fopen(argv[3], "wb");
      if (f == NULL)
        {
          fprintf(stderr, "dagsearch: cannot create file %s\n", argv[3]);
          exit(11);
        }
      if (fwrite(cost, mrec+1, 1, f) != 1)
        {
          fprintf(stderr, "dagsearch: cannot write to file %s\n", argv[3]);
          exit(12);
        }
      if (fclose(f))
        {
          fprintf(stderr, "dagsearch: cannot close file %s\n", argv[3]);
          exit(13);
        }
@


1.9
log
@Rsultats stocks incrmentalement.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.8 2001/05/04 00:26:33 lefevre Exp lefevre $
a61 2
  {
    if (i == q)
d63 41
a103 34
    next:
      s[i]++;
      x[2*i+r[i]] <<= 1;
      v[i] = o[i] ? DIST(x[2*i], x[2*i+1]) : x[2*i] + x[2*i+1];
    }
    else
    {
      i++;
      x[2*i] = v[dag[2*i]];
      x[2*i+1] = v[dag[2*i+1]];
      o[i] = 0;
      r[i] = 0;
      s[i] = 0;
      v[i] = x[2*i] + x[2*i+1];
    }
    while (v[i] > mmax)
    {
      x[2*i] = v[dag[2*i]];
      x[2*i+1] = v[dag[2*i+1]];
      if (r[i] == 0 && s[i])
      {
        x[2*i+1] <<= 1;
        r[i] = 1;
        s[i] = 1;
        v[i] = o[i] ? DIST(x[2*i], x[2*i+1]) : x[2*i] + x[2*i+1];
      }
      else if (o[i] == 0)
      {
        o[i] = 1;
        r[i] = 0;
        s[i] = 0;
        v[i] = DIST(x[2*i], x[2*i+1]);
      }
      else if (--i)
d105 21
a125 24
      else
        return;
    }
    if (v[i] == 0)
      goto next;
    if (i < q)
    {
      int j;
      for (j = 0; j < i; j++)
      {
        VALUE x;
        x = v[j];
        while (x < v[i])
          x <<= 1;
        if (x == v[i])
          goto next;
      }
    }
    OUT("%3d: %3" VALUEFMT "  ( %3" VALUEFMT " %3" VALUEFMT " )",
        i, v[i], x[2*i], x[2*i+1]);
    if (v[i] <= mrec && cost[v[i]] > i)
    {
      cost[v[i]] = i;
      OUT("   -   Cost(%3" VALUEFMT ") = %d", v[i], i);
a126 2
    OUT("\n");
  }
d136 4
a139 4
  {
    fprintf(stderr, "Usage: dagsearch <mrec> <mmax> <file>\n");
    exit(1);
  }
d143 4
a146 4
  {
    fprintf(stderr, "dagsearch: mrec is too large (out of range)\n");
    exit(3);
  }
d148 4
a151 4
  {
    fprintf(stderr, "dagsearch: mrec must be at least 1\n");
    exit(2);
  }
d155 4
a158 4
  {
    fprintf(stderr, "dagsearch: mmax is too large (out of range)\n");
    exit(5);
  }
d160 4
a163 4
  {
    fprintf(stderr, "dagsearch: mmax must be greater or equal to mrec\n");
    exit(4);
  }
d167 4
a170 4
  {
    fprintf(stderr, "dagsearch: out of memory!\n");
    exit(10);
  }
a175 60
  {
    VALUE v;
    int q;
    int dag[2*QMAX];
    char buffer[8];
    FILE *f;

    line++;
    for (q = 0; scanf("%1[\n]", buffer) == 0; q++)
    {
      if (q > QMAX)
      {
        fprintf(stderr, "dagsearch: too many nodes line %lu\n", line);
        exit(6);
      }
      if (scanf("(%7[(),0-9])", buffer) != 1 ||
          sscanf(buffer, "%d,%d", dag+2*q, dag+2*q+1) != 2)
      {
        fprintf(stderr, "dagsearch: input error line %lu (bad format?)\n",
                line);
        exit(7);
      }
      scanf("%*[\t ]");
    }

    if (!q)
    {
      if (feof(stdin))
        return 0;
      fprintf(stderr, "dagsearch: input error line %lu (bad format?)\n",
              line);
      exit(8);
    }

    dagsearch(q, dag, cost, mrec, mmax);

    for (v = 1; v < mrec; v += 2)
    {
      VALUE w;
      int c;

      c = cost[v];
      w = v;
      while ((w <<= 1) <= mrec)
        if (cost[w] > c)
          cost[w] = c;
    }

    f = fopen(argv[3], "wb");
    if (f == NULL)
    {
      fprintf(stderr, "dagsearch: cannot create file\n");
      exit(11);
    }
    if (fwrite(cost, mrec+1, 1, f) != 1)
    {
      fprintf(stderr, "dagsearch: cannot write to file\n");
      exit(12);
    }
    if (fclose(f))
d177 62
a238 2
      fprintf(stderr, "dagsearch: cannot close file\n");
      exit(13);
a239 1
  }
@


1.8
log
@Flush de stdout.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.7 2001/05/03 13:07:12 lefevre Exp lefevre $
d131 1
a131 1
  VALUE mrec, mmax, v;
a133 1
  FILE *f;
d172 2
d177 1
d181 1
d204 1
a204 1
        break;
a210 1
  }
d212 4
a215 6
  cost[0] = 0;
  cost[1] = 0;
  for (v = 1; v < mrec; v += 2)
  {
    VALUE w;
    int c;
d217 6
a222 6
    c = cost[v];
    w = v;
    while ((w <<= 1) <= mrec)
      if (cost[w] > c)
        cost[w] = c;
  }
d224 16
a239 5
  f = fopen(argv[3], "wb");
  if (f == NULL)
  {
    fprintf(stderr, "dagsearch: cannot create file\n");
    exit(11);
a240 12
  if (fwrite(cost, mrec+1, 1, f) != 1)
  {
    fprintf(stderr, "dagsearch: cannot write to file\n");
    exit(12);
  }
  if (fclose(f))
  {
    fprintf(stderr, "dagsearch: cannot close file\n");
    exit(13);
  }

  return 0;
@


1.7
log
@Mise  jour. Stockage des rsultats...
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.6 2001/05/02 02:51:28 lefevre Exp lefevre $
d53 1
@


1.6
log
@Correction concernant la rduction de l'arbre de recherche.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.5 2001/05/02 02:49:32 lefevre Exp lefevre $
d14 1
d29 6
d37 1
a37 1
void dagsearch(int q, int *dag, long mmax)
d117 8
a124 4
#ifndef NDEBUG
    printf("%3d: %3" VALUEFMT "  ( %3" VALUEFMT " %3" VALUEFMT " )\n",
           i, v[i], x[2*i], x[2*i+1]);
#endif
d130 1
a130 1
  VALUE mrec, mmax;
d132 2
d165 8
d206 32
a237 1
    dagsearch(q, dag, mmax);
@


1.5
log
@Type des valeurs: long ou long long (si LONGLONG dfini).
Rduction de l'arbre de recherche.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.4 2001/04/30 08:53:40 lefevre Exp lefevre $
d97 1
@


1.4
log
@Correction de bugs.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.3 2001/04/30 04:15:25 lefevre Exp lefevre $
d18 10
d34 1
a34 1
  long *x;
d39 2
a40 2
  long v[QMAX+1];  /* values */
  long vp[2*QMAX];
d97 12
d110 2
a111 1
    printf("%3d: %3ld  ( %3ld %3ld )\n", i, v[i], x[2*i], x[2*i+1]);
d118 1
a118 1
  long mrec, mmax;
d127 6
a132 1
  mrec = strtol(argv[1], (char **) NULL, 0);
d138 2
d142 2
a143 2
    fprintf(stderr, "dagsearch: mrec is too large (out of range)\n");
    exit(3);
a144 2

  mmax = strtol(argv[2], (char **) NULL, 0);
a148 5
  }
  if (errno == ERANGE)
  {
    fprintf(stderr, "dagsearch: mmax is too large (out of range)\n");
    exit(5);
@


1.3
log
@Mise  jour.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.2 2001/04/25 12:17:03 lefevre Exp lefevre $
d41 1
d78 1
a78 1
        v[i] = DIST(v[dag[2*i]], v[dag[2*i+1]]);
d85 5
a89 1
    printf("%3d  %ld\n", i, v[i]);
@


1.2
log
@Mise  jour.
@
text
@d2 1
a2 1
 * $Id: dagsearch.c 1.1 2001/04/19 13:54:55 lefevre Exp lefevre $
d18 2
a22 1
  int next;
d24 1
d29 2
a30 1
  long v[QMAX];  /* values */
d40 1
d42 1
a42 2
  next = 0;
  for (i = 1; ;)
d44 1
a44 1
    if (next)
d46 1
d49 1
a49 1
      v[i] = ;
d53 1
d61 1
a61 1
    while (v[i] > max)
d70 1
a70 1
        v[i] = o[i] ? ABS(x[2*i] - x[2*i+1]) : x[2*i] + x[2*i+1];
d77 1
a77 1
        v[i] = ABS(v[dag[2*i]] - v[dag[2*i+1]]);
d79 3
a81 1
      else if (--i == 0)
d84 1
@


1.1
log
@Initial revision
@
text
@d2 1
a2 1
 * $Id$
d18 1
a18 1
void dagsearch(int q, int *dag)
d21 7
d33 45
d148 1
a148 1
    dagsearch(q, dag);
@
