head	1.6;
access;
symbols;
locks; strict;
comment	@ * @;


1.6
date	2003.01.01.12.55.15;	author lefevre;	state Exp;
branches;
next	1.5;

1.5
date	2002.12.31.19.03.52;	author lefevre;	state Exp;
branches;
next	1.4;

1.4
date	2002.12.12.09.47.39;	author lefevre;	state Exp;
branches;
next	1.3;

1.3
date	2002.12.12.09.24.05;	author lefevre;	state Exp;
branches;
next	1.2;

1.2
date	2001.04.18.14.36.05;	author lefevre;	state Exp;
branches;
next	1.1;

1.1
date	2001.04.17.19.37.48;	author lefevre;	state Exp;
branches;
next	;


desc
@Gnration de DAGs reprsentant une multiplication par une constante.
@


1.6
log
@Plus de commentaires et optimisation mineure.
@
text
@/*
 * $Id: gendags.c 1.5 2002/12/31 19:03:52 lefevre Exp lefevre $
 *
 * In order to find the possible results (in an interval) that can be
 * obtained with q shift-and-add operations, this program generates a
 * superset S(q) of the DAGs satisfying:
 *   * The DAG has q+1 nodes, denoted from 0 to q, the node 0 being called
 *     the source node, and the node q being called the target node.
 *   * Each node has one or two parents, except the source node,
 *     which has no parents.
 *   * From each node, there exists a path leading to the target node
 *     (i.e., there are no useless nodes): each node except the target
 *     node has at least one child.
 *
 * A node different from the source node will be represented by a pair of
 * integers (x,y) such that x >= y, where x and y are the parents (x = y
 * if there is only one parent). Since we seek to return DAGs up to an
 * isomorphism, nodes will be ordered in such a way that between nodes
 * (x,y) and (x',y'), the smaller one in the lexicographic order will
 * come first. Moreover a simple isomorphism search will be performed;
 * for instance, in each example, only the first DAG will be returned:
 *   * Example 1:
 *     (0,0) (0,0) (1,0) (3,2)
 *     (0,0) (0,0) (2,0) (3,1)
 *   * Example 2:
 *     (0,0) (0,0) (2,1) (3,1)
 *     (0,0) (0,0) (2,1) (3,2)
 *   * Example 3:
 *     (0,0) (0,0) (1,1) (3,2)
 *     (0,0) (0,0) (2,2) (3,1)
 *
 * Usage: gendags <level> <q>
 *   q: number of nodes + 1 (= number of elementary operations)
 *   level:
 *     0 -> pairs must be ordered.
 *     1 -> level 0 + simple isomorphism search.
 *   e.g.
 *     $ diff <(./gendags 0 4) <(./gendags 1 4)
 *     3d2
 *     < (0,0) (0,0) (2,0) (3,1)
 *     6d4
 *     < (0,0) (0,0) (2,1) (3,2)
 *     8d5
 *     < (0,0) (0,0) (2,2) (3,1)
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <errno.h>

#define odd(i) ((i) & 1)
#define even(i) (!odd(i))

int readint(char *value, char *var, int min, int max)
{
  long tmp;
  char *end;

  tmp = strtol(value, &end, 10);
  if (*end != '\0' || errno == ERANGE || tmp < min || tmp > max)
    {
      fprintf(stderr, "gendags: bad input value %s (must be in [%d,%d])\n",
              var, min, max);
      exit(2);
    }
  return (int) tmp;
}

int main(int argc, char **argv)
{
  int level;
  int q;
  int *dag;

  if (argc != 3)
    {
      fprintf(stderr, "Usage: gendags <level> <q>\n");
      exit(1);
    }

  level = readint(argv[1], "level", 0, INT_MAX);
  q = readint(argv[2], "q", 1, 31);
  if (q == 1)
    {
      printf("(0,0)\n");
      return 0;
    }

  dag = calloc(2*q, sizeof(int));
  if (dag == NULL)
    {
      fprintf(stderr, "gendags: out of memory!\n");
      exit(5);
    }

  while (1)
    {
      unsigned long u;

    nextdag:
      do
        {
          int i, j, x, y;

          i = 2*q - 1;
          do
            {
              assert(odd(i) && dag[i] <= dag[i-1]);
              if (dag[i] != dag[i-1])
                {
                  y = dag[i] + 1;
                  x = dag[--i];
                  goto inc_ok;
                }
              i--;
              assert(even(i) && dag[i] <= i/2);
              if (dag[i] != i/2)
                {
                  x = dag[i] + 1;
                  y = 0;
                  goto inc_ok;
                }
              i--;
            }
          while (i > 2);
          return 0;

        inc_ok:
          assert(even(i));
          u = (1UL << x) | (1UL << y);
          for (j = 0; j < i; j++)
            u |= 1UL << dag[j];
          while (i < 2*q)
            {
              dag[i] = x;
              dag[i+1] = y;
              i += 2;
            }
        }
      while (u != (1UL << q) - 1);

      if (level)
        {
          int i;

          for (i = 0; i < q-2; i++)
            {
              int j, x, y;

              x = dag[2*i];
              y = dag[2*i+1];
              j = i;
              do
                {
                  j++;
                  assert(j < q);  /* because the last pair contains q-1 and
                                     is different from the other ones */
                }
              while (dag[2*j] == x && dag[2*j+1] == y);
              if (j - i > 1)  /* number of identical pairs */
                while (j < q)
                  {
                    unsigned long v;
                    v = ( ((1UL << dag[2*j]) | (1UL << dag[2*j+1]))
                          >> (i+1) ) & 3;
                    if (v == 1)  /* 1st pair referenced, not the 2nd -> OK */
                      break;
                    if (v == 2)  /* 2nd pair referenced, not the 1st -> rej */
                      goto nextdag;
                    j++;
                  }
            }  /* loop i */
        }  /* if (level) */

      {
        int i;
        for (i = 0; i < q; i++)
          printf("(%d,%d)%s", dag[2*i], dag[2*i+1], i < q - 1 ? " " : "\n");
      }
    }
}
@


1.5
log
@Correction d'un message d'erreur.
@
text
@d2 1
a2 1
 * $Id: gendags.c 1.4 2002/12/12 09:47:39 lefevre Exp lefevre $
d7 2
a8 2
 *   * The DAG has q+1 nodes, the node 0 being called the source
 *     node, and the node q being called the target node.
d11 20
a30 2
 *   * From each node, there exists a path leading to the target
 *     node (i.e., there are no useless nodes).
d36 1
a36 1
 *     1 -> level 0 + simple isomorphism search
d113 1
a113 1
                  y = ++dag[i];
d121 1
a121 1
                  x = ++dag[i];
@


1.4
log
@Code plus fiable (strtol  la place de atoi, etc.).
@
text
@d2 1
a2 1
 * $Id: gendags.c 1.3 2002/12/12 09:24:05 lefevre Exp lefevre $
d76 1
a76 1
      fprintf(stderr, "sdags: out of memory!\n");
@


1.3
log
@Style GNU.
@
text
@d2 1
a2 1
 * $Id: gendags.c 1.2 2001/04/18 14:36:05 lefevre Exp lefevre $
d32 2
d38 15
d65 2
a66 18
  level = atoi(argv[1]);
  if (level < 0)
    {
      fprintf(stderr, "gendags: level must not be negative\n");
      exit(2);
    }

  q = atoi(argv[2]);
  if (q < 1)
    {
      fprintf(stderr, "gendags: q must be at least 1\n");
      exit(3);
    }
  if (q > 32)
    {
      fprintf(stderr, "gendags: q must be at most 32\n");
      exit(4);
    }
d114 1
a114 1
          u = (1 << x) | (1 << y);
d116 1
a116 1
            u |= 1 << dag[j];
d148 2
a149 1
                    v = (((1 << dag[2*j]) | (1 << dag[2*j+1])) >> (i+1)) & 3;
@


1.2
log
@Recherche simple de DAGs isomorphes.
@
text
@d2 1
a2 1
 * $Id: gendags.c 1.1 2001/04/17 19:37:48 lefevre Exp lefevre $
d43 4
a46 4
  {
    fprintf(stderr, "Usage: gendags <level> <q>\n");
    exit(1);
  }
d50 4
a53 4
  {
    fprintf(stderr, "gendags: level must not be negative\n");
    exit(2);
  }
d57 4
a60 4
  {
    fprintf(stderr, "gendags: q must be at least 1\n");
    exit(3);
  }
d62 4
a65 4
  {
    fprintf(stderr, "gendags: q must be at most 32\n");
    exit(4);
  }
d67 4
a70 4
  {
    printf("(0,0)\n");
    return 0;
  }
d74 4
a77 4
  {
    fprintf(stderr, "sdags: out of memory!\n");
    exit(5);
  }
a79 5
  {
    unsigned long u;

  nextdag:
    do
d81 1
a81 1
      int i, j, x, y;
d83 1
a83 1
      i = 2*q - 1;
a84 3
      {
        assert(odd(i) && dag[i] <= dag[i-1]);
        if (dag[i] != dag[i-1])
d86 36
a121 3
          y = ++dag[i];
          x = dag[--i];
          goto inc_ok;
d123 3
a125 3
        i--;
        assert(even(i) && dag[i] <= i/2);
        if (dag[i] != i/2)
d127 29
a155 8
          x = ++dag[i];
          y = 0;
          goto inc_ok;
        }
        i--;
      }
      while (i > 2);
      return 0;
a156 6
    inc_ok:
      assert(even(i));
      u = (1 << x) | (1 << y);
      for (j = 0; j < i; j++)
        u |= 1 << dag[j];
      while (i < 2*q)
d158 3
a160 3
        dag[i] = x;
        dag[i+1] = y;
        i += 2;
a162 40
    while (u != (1UL << q) - 1);

    if (level)
    {
      int i;

      for (i = 0; i < q-2; i++)
      {
        int j, x, y;

        x = dag[2*i];
        y = dag[2*i+1];
        j = i;
        do
        {
          j++;
          assert(j < q);  /* because the last pair contains q-1 and
                           * is different from the other ones */
        }
        while (dag[2*j] == x && dag[2*j+1] == y);
        if (j - i > 1)  /* number of identical pairs */
          while (j < q)
          {
            unsigned long v;
            v = (((1 << dag[2*j]) | (1 << dag[2*j+1])) >> (i+1)) & 3;
            if (v == 1)  /* 1st pair referenced, not the 2nd -> OK */
              break;
            if (v == 2)  /* 2nd pair referenced, not the 1st -> reject */
              goto nextdag;
            j++;
          }
      }  /* loop i */
    }  /* if (level) */

    {
      int i;
      for (i = 0; i < q; i++)
        printf("(%d,%d)%s", dag[2*i], dag[2*i+1], i < q - 1 ? " " : "\n");
    }
  }
@


1.1
log
@Initial revision
@
text
@d2 1
a2 1
 * $Id$
d14 1
a14 1
 * Usage: gendags <q>
d16 11
d38 1
d42 1
a42 1
  if (argc != 2)
d44 1
a44 1
    fprintf(stderr, "Usage: gendags <q>\n");
d48 8
a55 1
  q = atoi(argv[1]);
d59 1
a59 1
    exit(2);
d64 1
a64 1
    exit(3);
d76 1
a76 1
    exit(4);
d83 1
d111 1
a111 1
      inc_ok:
d124 32
@
