/*
 * $Id: maxc.c 1.2 2003/03/30 13:07:40 lefevre Exp $
 *
 * Usage: maxc <file>
 * where the file was generated by dagsearch (or similar program)
 *
 * For each value of m:
 *   - maximum value of the cost, cinf or csup amongst
 *     all the m-bit constants,
 *   - smallest m-bit constant having this maximum value.
 */

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv)
{
  FILE *f;
  int m;
  unsigned long n;

  if (argc != 2)
    {
      fprintf(stderr, "Usage: maxc <file>\n");
      exit(1);
    }

  f = fopen(argv[1], "rb");
  if (f == NULL)
    {
      fprintf(stderr, "maxc: cannot open file %s\n", argv[1]);
      exit(2);
    }
  if (fseek(f, 2, SEEK_SET))
    {
      fprintf(stderr, "maxc: fseek(f, 2, SEEK_SET) failed!\n");
      exit(3);
    }

  for (m = 2, n = 2;;)
    {
      unsigned long n2, nc = 0;
      int cmax = 0;

      for (n2 = n << 1; n < n2; n++)
        {
          int c;

          c = getc(f);
          if (c == EOF)
            return 0;
          if (c > cmax)
            cmax = c, nc = n;
        }

      if (m > 2)
        putchar('\n');
      printf("Number of bits: %d\n  cmax = %d\n  n = %lu (",
             m++, cmax, nc);
      while (n2 >>= 1)
        putchar('0' + ((nc & n2) != 0));
      puts(")");
    }
}
