/*
 * $Id: readcode.c 1.1 2003/01/01 03:33:45 lefevre Exp $
 *
 * Codes corresponding to cinf and csup, read from the code file
 * generated by dagsearch. The dags are read from stdin.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

static int *dags = NULL;
static int dagslen = 0;
static int ndags = 0;

static void getdags(void)
{
  int s = 0;
  int i = 0;
  char buffer[8];

  while (scanf("%*[\t ]"), !feof(stdin))
    {
      if (ndags == s)
        {
          s = s ? 2 * s : 64;
          dags = realloc(dags, s * sizeof(*dags));
          if (dags == NULL)
            {
              fprintf(stderr, "dagsearch: out of memory!\n");
              exit(8);
            }
        }
      if (scanf("(%7[,0-9])%*[\t ]", buffer) != 1 ||
          sscanf(buffer, "%d,%d", dags+ndags, dags+ndags+1) != 2)
        {
          fprintf(stderr, "dagsearch: input error\n");
          exit(7);
        }
      ndags += 2;
      i += 2;
      if (scanf("%1[\n]", buffer) != 1)
        continue;
      if (dagslen == 0)
        dagslen = i;
      else if (i != dagslen)
        {
          fprintf(stderr, "dagsearch: the DAGs don't have a constant"
                  " length (%d and %d)\n", dagslen, i);
          exit(9);
        }
      i = 0;
    }
}

static int wcode(char *var, long int n, unsigned char *c)
{
  int q;
  int dag;
  int *p;
  int smax = 0;
  long long v[7];

  dag = *c;
  dag += *++c << 8;
  if (dag > ndags)
    {
      fprintf(stderr, "readcode: DAG number is too high (%d > %d)",
              dag, ndags);
      exit(10);
    }

  printf("%s (n = %ld):\n", var, n);
  p = dags + dagslen * (dag - 1);
  q = 0;
  v[0] = 1;
  while (*++c)
    {
      int x, y;
      int o, r, s;
      long long vx, vy;

      x = *(p++);
      y = *(p++);
      o = (*c) & 0x80;
      r = (*c) & 0x40;
      s = (*c) & 0x3f;
      if (s > smax)
        smax = s;
      vx = v[x];
      vy = v[y];
      if (r)
        vy <<= s;
      else
        vx <<= s;
      v[++q] = o ? vx - vy : vx + vy;
      if (v[q] < 0)
        {
          int tmp;
          v[q] = -v[q];
          r ^= 0x40;
          tmp = x;
          x = y;
          y = tmp;
        }
      printf("  u%d = u%d", q, x);
      if (!r)
        printf(" << %d", s);
      printf(" %c u%d", o ? '-' : '+', y);
      if (r)
        printf(" << %d", s);
      putchar('\n');
    }

  if (v[q] != n)
    {
      fprintf(stderr, "readcode: computed value is incorrect\n");
      exit(11);
    }

  printf("--> %s = %d\n", var, smax);

  return q;
}

int main(int argc, char **argv)
{
  FILE *f;
  int i;

  if (argc < 2)
    {
      fprintf(stderr, "Usage: readcode <file> [ <n> ... ]\n");
      exit(1);
    }

  getdags();

  f = fopen(argv[1], "rb");
  if (f == NULL)
    {
      fprintf(stderr, "readcode: cannot open file %s\n", argv[1]);
      exit(2);
    }

  for (i = 2; i < argc; i++)
    {
      long int n;
      char *end;
      unsigned char c[16];
      int q;

      if (i > 2)
        putchar('\n');
      n = strtol(argv[i], &end, 0);
      if (*end != '\0' || errno == ERANGE || n < 3 || (n & 1) == 0)
        {
          fprintf(stderr, "readcode: bad value %s\n", argv[i]);
          exit(3);
        }
      if (fseek(f, 8 * (n - 1), SEEK_SET))
        {
          fprintf(stderr, "readcode: fseek failed!\n");
          exit(4);
        }
      if (fread(c, 16, 1, f) != 1)
        {
          fprintf(stderr, "readcode: fread failed!\n");
          exit(5);
        }
      q = wcode("cinf", n, c);
      if (wcode("csup", n, c+8) != q)
        {
          fprintf(stderr, "readcode: costs for cinf and csup don't match\n");
          exit(6);
        }
    }

  return 0;
}
